#ifndef LIBWIISPRITE_SPRITE
#define LIBWIISPRITE_SPRITE

#include <stdlib.h>
#include <gccore.h>
#include "image.h"
#include "layer.h"
#include "tiledlayer.h"

//!libwiisprite namespace
namespace wsp{
	//!Basic data for a rectangle.
	struct Rectangle{
		f32 x, //!<X position.
			y; //!<Y position.
		f32 width, //!<Width of rectangle.
			height; //!<Height of rectangle.
	};

	//!Texture transformation.
	enum TRANSFORM{
		TRANS_NONE = 0, //!< No transformation.
		TRANS_MIRROR //!< Texture is mirrored.
	};

	//!A basic drawable object with animation support.
	class Sprite : public Layer{
		public:
			//!Constructor.
			Sprite();
			//!Destructor.
			virtual ~Sprite();
			
			//!Assigns this sprite to an image.
			//!If there is already an image with the same size, no data gets changed.
			//!\param image The image for this sprite.
			//!\param frameWidth The width of the frame. Should be a multiple of image->GetWidth() or 0 if it should get the same width as the image.
			//!\param frameHeight The height of the frame. Should be a multiple of image->GetHeight() or 0 if it should get the same height as the image.
			void SetImage(Image* image, u32 frameWidth = 0, u32 frameHeight = 0);
			//!Gets the assigned image.
			//!\return A pointer to the image. NULL if no image was assigned.
			const Image* GetImage() const;
			//!Changes the transformation of the texture.
			//!\param transform The new transformation for the texture.
			void SetTransform(TRANSFORM transform);
			//!Gets the texture transformation.
			//!\return The transformation of the texture.
			TRANSFORM GetTransform() const;

			//!Sets the rotation angle of the sprite.
			//!\param rotation The new angle of the sprite.
			void SetRotation(f32 rotation);
			//!Gets the rotation angle of the sprite.
			//!\return The current angle of the sprite.
			f32 GetRotation() const;

			//!Sets the zooming of the sprite.
			//!\param zoom The new zoom of the sprite. 1 is normal size, cannot be smaller than 0.
			void SetZoom(f32 zoom);
			//!Gets the zooming of the sprite.
			//!\return The current zoom of the sprite. 1 is normal size.
			f32 GetZoom() const;

			//!Sets the transparency of the sprite. 
			//!\param alpha Sets the transparency. Has a range from 0x00 (invisible) to 0xFF (fully visible)
			void SetTransparency(u8 alpha);
			//!Gets the transparency of the sprite.
			//!\return The current transparency of the sprite. Has a range from 0x00 (invisible) to 0xFF (fully visible)
			u8 GetTransparency() const;
			

			//!Defines a collision rectangle. On startup it's the same as Image width and height.
			//!\param x Offset from the upper left corners position of the sprite.
			//!\param y Offset from the upper left corners position of the sprite.
			//!\param width The width of the collision rectangle.
			//!\param height The height of the collision rectangle.
			void DefineCollisionRectangle(f32 x, f32 y, f32 width, f32 height);
			//!Gets the current collision rectangle.
			//!\return A pointer to the rectangle.
			const Rectangle* GetCollisionRectangle() const;
			//!Checks if another rectangle collides with this sprite.
			//!\param rect The rectangle to check.
			//!\param x The position of the rectangle, since x in rectangle is only an offset.
			//!\param y The position of the rectangle, since y in rectangle is only an offset.
			//!\return true if it is colliding, false if not.
			bool CollidesWith(const Rectangle* rect, f32 x = 0, f32 y = 0) const;
			//!Checks if another sprite collides with this sprite.
			//!\param sprite The sprite to check.
			//!\return true if it is colliding, false if not.
			bool CollidesWith(const Sprite* sprite) const;
			//!Checks if the sprite does cross a tile on the tiledlayer which number is not 0.
			//!\param tiledlayer the tiledlayer to check.
			//!\return true if it is colliding, false if not.
			bool CollidesWith(const TiledLayer* tiledlayer) const;
			
			//!Gets the current frame of the sprite.
			//!\return The frame this sprite is at.
			u32 GetFrame() const;
			//!Gets how long the current frame sequence is.
			//!Same as GetRawFrameCount() if no frame sequence speficied.
			u32 GetFrameSequenceLength() const;
			//!Gets how many frames there are at all.
			u32 GetRawFrameCount() const;
			//!Sets the current frame to a frame at the sequence.
			//!\param sequenceIndex The index in the current frame sequence.
			void SetFrame(u32 sequenceIndex);
			//!Sets the current frame to the next frame in the sequence. Goes back to the first member if no frames left in the sequence.
			void NextFrame();
			//!Sets the current frame to the previous frame in the sequence. Goes to the last member if the current frame is 0.
			void PrevFrame();
			//!Sets the current frame sequence.
			//!\param sequence An array with data for each sequence. Each member cannot be bigger than the maximum amount of frames.
			//!\param length The length of the sequence. Gets length amount of data.
			void SetFrameSequence(u32* sequence, u32 length);

			//!Draws the Sprite.
			void Draw(f32 offsetX = 0, f32 offsetY = 0) const;
		protected:
		private:
			void _CalcFrame();

			f32 _rotation, _zoom;
			u8 _alpha;
			Image* _image;
			TRANSFORM _trans;

			Rectangle* _colRect;

			u32 _frame, _frameRawCount;
			u32* _frameSeq; u32 _frameSeqLength, _frameSeqPos;
			f32 _frameX, _frameY;
	};
};

#endif
